/*
 *  pedo.c
 *  pedo
 *
 *  Created by Maurice Castro on 9/04/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

/*
 * Copyright 2010 Maurice Castro. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials provided
 *        with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Maurice Castro ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Maurice Castro OR OTHER CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "pedo.h"

#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOCFPlugIn.h>

#include <IOKit/usb/IOUSBLib.h>

#include <unistd.h>
int *r = NULL;
char *errmsg = NULL; 
Boolean foundOnce  = false;

IOReturn readDevice(IOUSBInterfaceInterface245 **intf)
{
	
	IOUSBDevRequest req;
	IOReturn err;
	
	int8_t buf[] = {0x55};
	int8_t buf2[128][8];
	int i;
	UInt32 sz;

	r = calloc(129, sizeof(r));
	r[128] = -1;
	if (!r)
	{
		errmsg = "Calloc failed";
		exit(1);
	}
	
    req.bmRequestType = 0x40;
    req.bRequest = 0x6; 
    req.wValue = 0;
    req.wIndex = 0; 
    req.wLength = 1; 
    req.pData = buf;
    
    err = (*intf)->ControlRequest(intf, 0, &req);
    if (kIOReturnSuccess == err) 
    {
        if(req.wLenDone != 1)
        {
			return(kIOReturnUnderrun);
        }
    }
	for (i=0; i < 128; i++)
	{
		sz = 8;
		if ((*intf)->ReadPipe(intf, 1, buf2[i], &sz) != kIOReturnSuccess)
		{
			errmsg = "Read failed";
			return(-1);
		}
	}
	for (i = 0; i < 128; i++)
	{
		if (buf2[i][1] == -1 && buf2[i][0] == -1)
			r[i] = -1;
		else
			r[i] = (u_int8_t) buf2[i][1] * 256 + (u_int8_t) buf2[i][0];
	}
	
    return(err);
}

IOReturn clearDevice(IOUSBInterfaceInterface245 **intf)
{
	
	IOUSBDevRequest req;
	IOReturn err;
	
	int8_t buf[] = {0xaa};
	
    req.bmRequestType = 0x40;
    req.bRequest = 0x6; 
    req.wValue = 0;
    req.wIndex = 0; 
    req.wLength = 1; 
    req.pData = buf;
    
    err = (*intf)->ControlRequest(intf, 0, &req);
    if (kIOReturnSuccess == err) 
    {
        if(req.wLenDone != 1)
        {
			return(kIOReturnUnderrun);
        }
    }
    return(err);
}

void useUSBInterface(IOUSBInterfaceInterface245 **intf)
{
    /* put code to read device here and to reset device */
	/* finallyDoSomethingWithThisDevice(intf); */
	readDevice(intf);
	clearDevice(intf);
}

UInt32 openUSBInterface(IOUSBInterfaceInterface245 **intf)
{
	IOReturn ret;
	
	ret = (*intf)->USBInterfaceOpen(intf);
    if(ret != kIOReturnSuccess)
    {
        errmsg = "Could not set configuration on device";
        return(-1);
    }
	
    return(0);
}

IOUSBInterfaceInterface245 **getUSBInterfaceInterface(io_service_t usbInterface)
{
	IOReturn err;
	IOCFPlugInInterface **plugInInterface=NULL;
	IOUSBInterfaceInterface245 **intf=NULL;
	SInt32 score;
	HRESULT res;
	
    // There is no documentation for IOCreatePlugInInterfaceForService or QueryInterface, you have to use sample code.
	
	err = IOCreatePlugInInterfaceForService(usbInterface, 
											kIOUSBInterfaceUserClientTypeID, 
											kIOCFPlugInInterfaceID,
											&plugInInterface, 
											&score);
    (void)IOObjectRelease(usbInterface);				// done with the usbInterface object now that I have the plugin
    if ((kIOReturnSuccess != err) || (plugInInterface == nil) )
    {
        errmsg = "Unable to create plug in interface for USB interface";
        return(nil);
    }
    
    res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID245), (LPVOID)&intf);
    IODestroyPlugInInterface(plugInInterface);			// done with this
	
    if (res || !intf)
    {
        errmsg = "Unable to create interface with QueryInterface";
        return(nil);
    }
    return(intf);
}

Boolean isThisTheInterfaceYoureLookingFor(IOUSBInterfaceInterface245 **intf)
{
    //	Check to see if this is the interface you're interested in
    //  This code is only expecting one interface, so returns true
    //  the first time.
    //  You code could check the nature and type of endpoints etc
	
    if(foundOnce)
    {
			return(false);
    }
    foundOnce = true;
    return(true);
}


int iterateinterfaces(io_iterator_t interfaceIterator)
{
	io_service_t usbInterface;
	int err = 0;
	IOReturn ret;
	IOUSBInterfaceInterface245 **intf=NULL;
	
    usbInterface = IOIteratorNext(interfaceIterator);
    if(usbInterface == IO_OBJECT_NULL)
    {
        errmsg = "Unable to find an Interface";
        return(-1);
    }
    
    while(usbInterface != IO_OBJECT_NULL)
    {
        intf = getUSBInterfaceInterface(usbInterface);
        
        if(intf != nil)
        {
			// Don't release the interface here. That's one too many releases and causes set alt interface to fail
            if(isThisTheInterfaceYoureLookingFor(intf))
            {
                err = openUSBInterface(intf);
                if(err == 0)
                {
                    useUSBInterface(intf);
                    ret = (*intf)->USBInterfaceClose(intf);
                }
				
                ret = (*intf)->Release(intf);
                // Not worth bothering with errors here
                return(err);
            }
        }
        usbInterface = IOIteratorNext(interfaceIterator);
    }
	
    errmsg = "Device Not Found";
    IOObjectRelease(usbInterface);
    return(-1);
}

void useUSBDevice(IOUSBDeviceInterface245 **dev, UInt32 configuration)
{
	io_iterator_t interfaceIterator;
	IOUSBFindInterfaceRequest req;
	IOReturn err;
	
    err = (*dev)->SetConfiguration(dev, configuration);
    if(err != kIOReturnSuccess)
    {
        errmsg = "Could not set configuration on device";
        return;
    }
	
    req.bInterfaceClass = kIOUSBFindInterfaceDontCare;
    req.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
    req.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
    req.bAlternateSetting = kIOUSBFindInterfaceDontCare;
    
    err = (*dev)->CreateInterfaceIterator(dev, &req, &interfaceIterator);
    if(err != kIOReturnSuccess)
    {
        errmsg = "Could not create interface iterator";
        return;
    }
	
    
    err = iterateinterfaces(interfaceIterator);
	
    IOObjectRelease(interfaceIterator);
	
	
}


SInt32 openUSBDevice(IOUSBDeviceInterface245 **dev)
{
	UInt8 numConfig;
	IOReturn err;
	IOUSBConfigurationDescriptorPtr desc;
	
    err = (*dev)->GetNumberOfConfigurations(dev, &numConfig);
    if(err != kIOReturnSuccess)
    {
        errmsg = "Could not get number of configurations from device";
        return(-1);
    }
    if(numConfig != 1)
    {
        errmsg = "Wrong number of configurations";
        return(-1);
    }
    
    err = (*dev)->GetConfigurationDescriptorPtr(dev, 0, &desc);
    if(err != kIOReturnSuccess)
    {
        errmsg = "Could not get configuration descriptor from device";
        return(-1);
    }
	
    
    // We should really try to do classic arbitration here
    
    err = (*dev)->USBDeviceOpen(dev);
    if(err == kIOReturnExclusiveAccess)
    {
        return(-2);
    }
    if(err != kIOReturnSuccess)
    {
        errmsg = "Could not open device";
        return(-1);
    }
    
    
    return(desc->bConfigurationValue);
}


IOUSBDeviceInterface245 **getUSBDevice(io_object_t usbDevice)
{
	IOReturn err;
	IOCFPlugInInterface **plugInInterface=NULL;
	IOUSBDeviceInterface245 **dev=NULL;
	SInt32 score;
	HRESULT res;
	
    // There is no documentation for IOCreatePlugInInterfaceForService or QueryInterface, you have to use sample code.
	
	err = IOCreatePlugInInterfaceForService(usbDevice, 
											kIOUSBDeviceUserClientTypeID, 
											kIOCFPlugInInterfaceID,
											&plugInInterface, 
											&score);
    if ((kIOReturnSuccess != err) || (plugInInterface == nil) )
    {
        errmsg = "Unable to create plug in interface for USB device";
        return(nil);
    }
    
    res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID245), (LPVOID)&dev);
    IODestroyPlugInInterface(plugInInterface);			// done with this
	
    if (res || !dev)
    {
        errmsg = "Unable to create USB device with QueryInterface";
        return(nil);
    }
	
    return(dev);
}

int iterateDevices(io_iterator_t deviceIterator)
{
	io_object_t usbDevice;
	int err = -1;
	IOReturn ret;
	IOUSBDeviceInterface245 **dev=NULL;
	SInt32 config = 0;
	int exclusiveErrs, attempts;
	
	
    for(attempts = 1; attempts < 5; attempts++)
    {
        exclusiveErrs = 0;
        usbDevice = IOIteratorNext(deviceIterator);
        if(usbDevice == IO_OBJECT_NULL)
        {
            errmsg = "Unable to find first matching USB device";
            return(-1);
        }
        
        while(usbDevice != IO_OBJECT_NULL)
        {
            dev = getUSBDevice(usbDevice);
            
            if(dev != nil)
            {
                
                config = openUSBDevice(dev);
                if(config == -2)
                {
                    exclusiveErrs++;
                }
                else if(config >= 0)
                {
                    // Device sucessfully opened
					
                    if(config > 0)
                    {
                        useUSBDevice(dev, config);
                    }
                    else
                    {
                        errmsg = "zero configurations";
                    }
					
                    ret = (*dev)->USBDeviceClose(dev);
                }
				
                ret = (*dev)->Release(dev);
                // Not worth bothering with errors here
            }
            IOObjectRelease(usbDevice);
            
            if(config >= 0)	// we have sucessfully used device 
            {
                return(0);
            }
            usbDevice = IOIteratorNext(deviceIterator);
        };
        if(exclusiveErrs > 0)
        {
	     	sleep(1);
            IOIteratorReset(deviceIterator);
        }
        else
        {
            break;
        }
    }
    return(err);
}


int *dreamCheekyPedo(char **errormsg)
{
	IOReturn err;
	mach_port_t masterPort;
	CFMutableDictionaryRef dict;
	SInt32 usbVID;
	SInt32 usbPID;
	
	SInt32 usbConfig;
	SInt32 usbIntNum;
	
	io_iterator_t anIterator;
	
    usbVID = 0x04f3;
    usbPID = 0xa003;	
    
	r = NULL;
	errmsg = NULL;
	foundOnce  = false;
	
    err = IOMasterPort(MACH_PORT_NULL, &masterPort);
    if(err != kIOReturnSuccess)
    {
        *errormsg = "Could not get master port";
        return(NULL);
    }
    
    dict = IOServiceMatching("IOUSBInterface");
	
    if(dict == nil)
    {
        *errormsg =  "Could not create matching dictionary";
        return(NULL);
    }
	
	CFDictionarySetValue(
						 dict, 
						 CFSTR(kUSBVendorID), 
						 CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbVID)
						 );
	CFDictionarySetValue(dict, CFSTR(kUSBProductID), 
						 CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbPID));
	
    
    usbConfig = 1;
	usbIntNum = 0;
	CFDictionarySetValue(dict, CFSTR(kUSBConfigurationValue), 
						 CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbConfig));
	CFDictionarySetValue(dict, CFSTR(kUSBInterfaceNumber), 
						 CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbIntNum));
	
	err = IOServiceGetMatchingServices(masterPort, dict, &anIterator);
    if(err != kIOReturnSuccess)
    {
        *errormsg = "Could not get device iterator";
        return(NULL);
    }
    
    err = iterateinterfaces(anIterator);
	
	*errormsg = errmsg;
    
    IOObjectRelease(anIterator);
	return(r);
}
